diff -U5 -ur nginx-0.6.32-orig/src/http/modules/ngx_http_proxy_module.c nginx-0.6.32/src/http/modules/ngx_http_proxy_module.c
--- nginx-0.6.32-orig/src/http/modules/ngx_http_proxy_module.c	2008-07-07 18:18:41.000000000 +0800
+++ nginx-0.6.32/src/http/modules/ngx_http_proxy_module.c	2009-08-10 12:23:30.000000000 +0800
@@ -156,10 +156,11 @@
     { ngx_string("timeout"), NGX_HTTP_UPSTREAM_FT_TIMEOUT },
     { ngx_string("invalid_header"), NGX_HTTP_UPSTREAM_FT_INVALID_HEADER },
     { ngx_string("http_500"), NGX_HTTP_UPSTREAM_FT_HTTP_500 },
     { ngx_string("http_503"), NGX_HTTP_UPSTREAM_FT_HTTP_503 },
     { ngx_string("http_404"), NGX_HTTP_UPSTREAM_FT_HTTP_404 },
+    { ngx_string("http_403"), NGX_HTTP_UPSTREAM_FT_HTTP_403 },
     { ngx_string("off"), NGX_HTTP_UPSTREAM_FT_OFF },
     { ngx_null_string, 0 }
 };
 
 
diff -U5 -ur nginx-0.6.32-orig/src/http/ngx_http_upstream.c nginx-0.6.32/src/http/ngx_http_upstream.c
--- nginx-0.6.32-orig/src/http/ngx_http_upstream.c	2008-04-17 22:32:11.000000000 +0800
+++ nginx-0.6.32/src/http/ngx_http_upstream.c	2009-08-10 12:22:53.000000000 +0800
@@ -1211,10 +1211,20 @@
             ngx_http_upstream_finalize_request(r, u, NGX_HTTP_NOT_FOUND);
             return;
         }
     }
 
+    if (u->headers_in.status_n == NGX_HTTP_FORBIDDEN) {
+
+        if (u->peer.tries > 1
+            && u->conf->next_upstream & NGX_HTTP_UPSTREAM_FT_HTTP_403)
+        {
+            ngx_http_upstream_next(r, u, NGX_HTTP_UPSTREAM_FT_HTTP_403);
+            return;
+        }
+    }
+
 
     if (u->headers_in.status_n >= NGX_HTTP_BAD_REQUEST
         && u->conf->intercept_errors)
     {
         clcf = ngx_http_get_module_loc_conf(r, ngx_http_core_module);
@@ -2216,11 +2226,11 @@
 
 #if 0
     ngx_http_busy_unlock(u->conf->busy_lock, &u->busy_lock);
 #endif
 
-    if (ft_type == NGX_HTTP_UPSTREAM_FT_HTTP_404) {
+    if ((ft_type == NGX_HTTP_UPSTREAM_FT_HTTP_404) || (ft_type == NGX_HTTP_UPSTREAM_FT_HTTP_403)) {
         state = NGX_PEER_NEXT;
     } else {
         state = NGX_PEER_FAILED;
     }
 
@@ -2248,10 +2258,14 @@
             break;
 
         case NGX_HTTP_UPSTREAM_FT_HTTP_404:
             status = NGX_HTTP_NOT_FOUND;
             break;
+        
+        case NGX_HTTP_UPSTREAM_FT_HTTP_403:
+            status = NGX_HTTP_FORBIDDEN;
+            break;
 
         /*
          * NGX_HTTP_UPSTREAM_FT_BUSY_LOCK and NGX_HTTP_UPSTREAM_FT_MAX_WAITING
          * never reach here
          */
diff -U5 -ur nginx-0.6.32-orig/src/http/ngx_http_upstream.h nginx-0.6.32/src/http/ngx_http_upstream.h
--- nginx-0.6.32-orig/src/http/ngx_http_upstream.h	2008-03-04 05:04:06.000000000 +0900
+++ nginx-0.6.32/src/http/ngx_http_upstream.h	2009-08-10 12:09:53.000000000 +0800
@@ -22,10 +22,11 @@
 #define NGX_HTTP_UPSTREAM_FT_HTTP_500        0x00000010
 #define NGX_HTTP_UPSTREAM_FT_HTTP_503        0x00000020
 #define NGX_HTTP_UPSTREAM_FT_HTTP_404        0x00000040
 #define NGX_HTTP_UPSTREAM_FT_BUSY_LOCK       0x00000080
 #define NGX_HTTP_UPSTREAM_FT_MAX_WAITING     0x00000100
+#define NGX_HTTP_UPSTREAM_FT_HTTP_403        0x00000200
 #define NGX_HTTP_UPSTREAM_FT_NOLIVE          0x40000000
 #define NGX_HTTP_UPSTREAM_FT_OFF             0x80000000
 
 
 #define NGX_HTTP_UPSTREAM_INVALID_HEADER     40
